%{
#include <stdio.h>
#include <stdlib.h>

#define YY_DECL \
  int yylex (YYSTYPE *yylval, YYLTYPE *yylloc)

#define YYERROR_DECL \
  void yyerror (YYLTYPE *yylloc, int* nerrors, char const *s)

#define LOC_FPRINTF(File, Loc)				\
  fprintf (File, "%d.%d-%d.%d: ",			\
	   (Loc)->first_line, (Loc)->first_column,	\
	   (Loc)->last_line, (Loc)->last_column)
%}

%initial-action
{
  @$.first_column = @$.first_line = @$.last_column = @$.last_line = 0;
}

%locations
%pure-parser
%parse-param { int* nerrors }
%error-verbose
%debug

%token PLUS "+"
       MINUS "-"
       TIMES "*"
       DIV "/"
       <ival> INT "integer"
       CALC_EOF 0 "end of file"
%union
{
  int ival;
  float fval;
  const char* cp;
}

%{
  YY_DECL;
  YYERROR_DECL;
%}

%type <ival> line exp
%left "+" "-"
%left "*" "/"
%printer { fprintf (stderr, "%d", $$); } INT exp line
%%

start:
  start line      { printf ("%d\n", $2); }
| /* nothing. */
;

line:
  exp '\n'   { $$ = $1; }
| error '\n' { $$ = 42; yyerrok; }
;

exp:
  exp "+" exp	{ $$ = $1 + $3; }
| exp "-" exp	{ $$ = $1 - $3; }
| exp "*" exp	{ $$ = $1 * $3; }
| exp "/" exp
{ 
  if ($3)
    $$ = $1 / $3;
  else
  {
    LOC_FPRINTF (stderr, &@3);
    fprintf (stderr, "invalid operand: %d\n", $3);
    $$ = 666;
  }
}
| '(' exp ')'	{ $$ = $2; }
| INT		{ $$ = $1; }
;

%%

YYERROR_DECL
{
  LOC_FPRINTF (stderr, yylloc);
  fprintf (stderr, "%s\n", s);
  ++*nerrors;
}

int
main ()
{
  int nerrors = 0;
  yydebug = !!getenv ("PARSE");
  yyparse (&nerrors);
  return !!nerrors;
}
